@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 2002-2011 Karl J. Runge <runge@karlrunge.com> 
+   Copyright (C) 2002-2013 Karl J. Runge <runge@karlrunge.com> 
    All rights reserved.
 
 This file is part of x11vnc.
@@ -278,6 +278,10 @@ char *ident_username(rfbClientPtr client) {
 		int block = 0;
 		int refused = 0;
 
+		if (getenv("X11VNC_NO_IDENT_USERNAME")) {
+			goto skip_ident;
+		}
+
 		/*
 		 * need to check to see if the operation will block for
 		 * a long time: a firewall may just ignore our packets.
@@ -386,6 +390,7 @@ char *ident_username(rfbClientPtr client) {
 				close(sock);
 			}
 		}
+		skip_ident:
 		if (! ok || !newuser) {
 			newuser = strdup("unknown-user");
 		}
