@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 2002-2011 Karl J. Runge <runge@karlrunge.com> 
+   Copyright (C) 2002-2013 Karl J. Runge <runge@karlrunge.com> 
    All rights reserved.
 
 This file is part of x11vnc.
@@ -1224,6 +1224,7 @@ void switch_to_xkb_if_better(void) {
 		    " characters.\n");
 	}
 	rfbLog("\n");
+	fflush(stderr);
 }
 
 /* sets up all the keymapping info via Xkb API */
@@ -2867,7 +2868,6 @@ static void pipe_keyboard(rfbBool down, rfbKeySym keysym, rfbClientPtr client) {
 	int can_input = 0, uid = 0, isbutton = 0;
 	allowed_input_t input;
 	char *name;
-	ClientData *cd = (ClientData *) client->clientData;
 
 	apply_remap(&keysym, &isbutton);
 
@@ -2931,8 +2931,12 @@ static void pipe_keyboard(rfbBool down, rfbKeySym keysym, rfbClientPtr client) {
 			can_input = 1;	/* XXX distinguish later */
 		}
 	}
-	if (cd) {
-		uid = cd->uid;
+
+	if (client) {
+		ClientData *cd = (ClientData *) client->clientData;
+		if (cd) {
+			uid = cd->uid;
+		}
 	}
 	if (! can_input) {
 		uid = -uid;
