@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 2002-2011 Karl J. Runge <runge@karlrunge.com> 
+   Copyright (C) 2002-2013 Karl J. Runge <runge@karlrunge.com> 
    All rights reserved.
 
 This file is part of x11vnc.
@@ -132,21 +132,29 @@ void macosxCG_refresh_callback_off(void) {
 
 extern int macosx_noscreensaver;
 extern int macosx_read_opengl;
+extern int macosx_read_cglion;
 extern int macosx_read_rawfb;
 
 extern void macosxGCS_initpb(void);
-extern int macosxCGP_init_dimming(void);
-extern int macosxCGP_undim(void);
-extern int macosxCGP_dim_shutdown(void);
+extern int  macosxCGP_init_dimming(void);
+extern int  macosxCGP_undim(void);
+extern int  macosxCGP_dim_shutdown(void);
 extern void macosxCGP_screensaver_timer_off(void);
 extern void macosxCGP_screensaver_timer_on(void);
+
 extern void macosx_opengl_init(void);
 extern void macosx_opengl_fini(void);
+extern void macosx_cglion_init(void);
+extern void macosx_cglion_fini(void);
 
 int x11vnc_macosx_no_deprecated_localevents = 0;
 int x11vnc_macosx_no_deprecated_postevents  = 0;
 int x11vnc_macosx_no_deprecated_framebuffer = 0;
 
+extern int macosx_display;
+extern int macosx_display_x;
+extern int macosx_display_y;
+
 void macosxCG_init(void) {
 
 	x11vnc_macosx_no_deprecated_localevents = 0;
@@ -164,13 +172,52 @@ void macosxCG_init(void) {
 	}
 
 	if (displayID == 0) {
+		unsigned int ndisp = 0;
+		CGDirectDisplayID disp[64];
 		macosx_log("macosxCG_init: initializing display.\n");
 
+		if (CGGetActiveDisplayList(64, disp, &ndisp) == kCGErrorSuccess) {
+			int i;
+			fprintf(stderr, "\nAvailable Displays:\n");
+			for (i=0; i < ndisp; i++) {
+				CGDirectDisplayID d = disp[i];
+				int w = CGDisplayPixelsWide(d);
+				int h = CGDisplayPixelsHigh(d);
+				int m = CGDisplayIsMain(d);
+				int a = CGDisplayIsActive(d);
+				int s = CGDisplayIsAsleep(d);
+				int b = CGDisplayIsBuiltin(d);
+				int o = CGDisplayUsesOpenGLAcceleration(d);
+				unsigned int vendor = CGDisplayVendorNumber(d);
+				CGSize sz = CGDisplayScreenSize(d);
+				CGRect bd = CGDisplayBounds(d);
+				fprintf(stderr, "%5d ID: 0x%08x width: %4d (%3d mm) height: %4d (%3d mm) at (%d, %d)\n	  Main=%d Builtin=%d Active=%d Sleep=%d OpenGL=%d Vendor: 0x%x%s\n",
+				    i, d, w, (int) sz.width, h, (int) sz.height, (int) bd.origin.x, (int) bd.origin.y, m, b, a, s, o, vendor, vendor == 0x610 ? " (Apple)" : "");
+			}
+			fprintf(stderr, "\n");
+		}
+
 		displayID = kCGDirectMainDisplay;
+
 #ifdef X11VNC_MACOSX_USE_GETMAINDEVICE
 		/* not sure this ever did anything. */
 		(void) GetMainDevice();
 #endif
+
+		if (macosx_display != 0 && ndisp > 0) {
+			int n = macosx_display;
+			if (n >= 0 && n < ndisp) {
+				CGRect bd;
+				displayID = disp[n];
+				bd = CGDisplayBounds(displayID);
+				macosx_display_x = (int) bd.origin.x;
+				macosx_display_y = (int) bd.origin.y;
+			} else {
+				fprintf(stderr, "macosxCG_init: invalid display: %d.  Must be 0 <= n < %d\n", n, ndisp);
+				exit(1);
+			}
+		}
+
 		if (displayID == 0) {
 			macosx_log("macosxCG_init: could not get kCGDirectMainDisplay / CGMainDisplayID() display.\n");
 			exit(1);
@@ -194,9 +241,13 @@ void macosxCG_init(void) {
 		}
 #endif
 
-		macosx_opengl_init();
+		macosx_cglion_init();
 
-		if (!macosx_read_opengl) {
+		if (!macosx_read_cglion) {
+			macosx_opengl_init();
+		}
+
+		if (!macosx_read_opengl && !macosx_read_cglion) {
 			char *addr = macosxCG_get_fb_addr();
 			if (addr == NULL) {
 				macosx_log("macosxCG_init: could not get raw framebuffer address / CGDisplayBaseAddress().\n");
@@ -222,6 +273,7 @@ void macosxCG_fini(void) {
 	}
 	macosxCG_refresh_callback_off();
 	macosx_opengl_fini();
+	macosx_cglion_fini();
 	displayID = 0;
 }
 
@@ -241,7 +293,7 @@ extern void clean_up_exit(int ret);
 
 void macosxCG_event_loop(void) {
 	OSStatus rc;
-	int nbpp;
+	int nbpp, nx, ny;
 	static int nbpp_save = -1;
 
 	macosxGCS_poll_pb();
@@ -258,6 +310,8 @@ void macosxCG_event_loop(void) {
 	}
 
 	nbpp = macosxCG_CGDisplayBitsPerPixel();
+	nx = macosxCG_CGDisplayPixelsWide();
+	ny = macosxCG_CGDisplayPixelsHigh();
 
 	if (nbpp_save < 0) {
 		nbpp_save = nbpp;
@@ -268,8 +322,13 @@ void macosxCG_event_loop(void) {
 		if (macosx_resize) {
 			do_new_fb(1);
 		}
-	} else if (wdpy_x != macosxCG_CGDisplayPixelsWide()) {
-	    if (wdpy_y != macosxCG_CGDisplayPixelsHigh()) {
+	} else if (wdpy_x != nx || wdpy_y != ny) {
+		if (nx == 0 || ny == 0) {
+			if (0 && macosx_read_cglion) {
+				macosx_log("nx=0 ny=0 but macosx_read_cglion\n");
+				return;
+			}
+		}
 		if (macosx_wait_for_switch) {
 			int cnt = 0;
 			while (1) {
@@ -285,17 +344,16 @@ void macosxCG_event_loop(void) {
 				}
 				sleep(1);
 			}
-			if (wdpy_x == macosxCG_CGDisplayPixelsWide()) {
-				if (wdpy_y == macosxCG_CGDisplayPixelsHigh()) {
-					macosx_log("we're back...\n");
-					return;
-				}
+			nx = macosxCG_CGDisplayPixelsWide();
+			ny = macosxCG_CGDisplayPixelsHigh();
+			if (wdpy_x == nx && wdpy_y == ny) {
+				macosx_log("we're back...\n");
+				return;
 			}
 		}
 		if (macosx_resize) {
 			do_new_fb(1);
 		}
-	    }
 	}
 	if (nbpp > 0) {
 		nbpp_save = nbpp;
@@ -303,6 +361,7 @@ void macosxCG_event_loop(void) {
 }
 
 extern int macosx_no_rawfb;
+
 extern int macosx_read_opengl;
 extern int macosx_opengl_get_width();
 extern int macosx_opengl_get_height();
@@ -310,28 +369,86 @@ extern int macosx_opengl_get_bpp();
 extern int macosx_opengl_get_bps();
 extern int macosx_opengl_get_spp();
 
+extern int macosx_read_cglion;
+extern int macosx_cglion_get_width();
+extern int macosx_cglion_get_height();
+extern int macosx_cglion_get_bpp();
+extern int macosx_cglion_get_bps();
+extern int macosx_cglion_get_spp();
+
+int macosx_get_width(void) {
+	if (macosx_read_opengl) {
+		return macosx_opengl_get_width();
+	}
+	if (macosx_read_cglion) {
+		return macosx_cglion_get_width();
+	}
+	return macosxCG_CGDisplayPixelsWide();
+}
+
+int macosx_get_height(void) {
+	if (macosx_read_opengl) {
+		return macosx_opengl_get_height();
+	}
+	if (macosx_read_cglion) {
+		return macosx_cglion_get_height();
+	}
+	return macosxCG_CGDisplayPixelsHigh();
+}
+
+int macosx_get_bpp(void) {
+	if (macosx_read_opengl) {
+		return macosx_opengl_get_bpp();
+	}
+	if (macosx_read_cglion) {
+		return macosx_cglion_get_bpp();
+	}
+	return macosxCG_CGDisplayBitsPerPixel();
+}
+
+int macosx_get_bps(void) {
+	if (macosx_read_opengl) {
+		return macosx_opengl_get_bps();
+	}
+	if (macosx_read_cglion) {
+		return macosx_cglion_get_bps();
+	}
+	return macosxCG_CGDisplayBitsPerSample();
+}
+
+int macosx_get_spp(void) {
+	if (macosx_read_opengl) {
+		return macosx_opengl_get_spp();
+	}
+	if (macosx_read_cglion) {
+		return macosx_cglion_get_spp();
+	}
+	return macosxCG_CGDisplaySamplesPerPixel();
+}
+
+int macosxCG_CGDisplayPixelsWide(void) {
+	return (int) CGDisplayPixelsWide(displayID);
+}
+int macosxCG_CGDisplayPixelsHigh(void) {
+	return (int) CGDisplayPixelsHigh(displayID);
+}
+
 #if X11VNC_MACOSX_NO_DEPRECATED_FRAMEBUFFER || X11VNC_MACOSX_NO_DEPRECATED
 
 char *macosxCG_get_fb_addr(void) {
 	return NULL;
 }
-int macosxCG_CGDisplayPixelsWide(void) {
-	return macosx_opengl_get_width();
-}
-int macosxCG_CGDisplayPixelsHigh(void) {
-	return macosx_opengl_get_height();
-}
 int macosxCG_CGDisplayBitsPerPixel(void) {
-	return macosx_opengl_get_bpp();
+	return 0;
 }
 int macosxCG_CGDisplayBitsPerSample(void) {
-	return macosx_opengl_get_bps();
+	return 0;
 }
 int macosxCG_CGDisplaySamplesPerPixel(void) {
-	return macosx_opengl_get_spp();
+	return 0;
 }
 int macosxCG_CGDisplayBytesPerRow(void) {
-	return macosx_opengl_get_width() * macosx_opengl_get_bpp() / 8;
+	return 0;
 }
 
 #else
@@ -341,52 +458,44 @@ char *macosxCG_get_fb_addr(void) {
 		macosx_log("CGDisplayBaseAddress disabled by env. var\n");
 		return NULL;
 	}
-	if (macosx_no_rawfb) {
-		macosx_log("CGDisplayBaseAddress disabled by user.\n");
-		return NULL;
-	}
 	if (macosx_read_opengl) {
 		macosx_log("CGDisplayBaseAddress disabled by OpenGL.\n");
 		return NULL;
 	}
+	if (macosx_read_cglion) {
+		macosx_log("CGDisplayBaseAddress disabled by CGLion.\n");
+		return NULL;
+	}
+	if (macosx_no_rawfb) {
+		macosx_log("CGDisplayBaseAddress disabled by user.\n");
+		return NULL;
+	}
 	return (char *) CGDisplayBaseAddress(displayID);
 }
 
-int macosxCG_CGDisplayPixelsWide(void) {
-	if ((0 && macosx_read_opengl) || x11vnc_macosx_no_deprecated_framebuffer) {
-		return macosx_opengl_get_width();
-	}
-	return (int) CGDisplayPixelsWide(displayID);
-}
-int macosxCG_CGDisplayPixelsHigh(void) {
-	if ((0 && macosx_read_opengl) || x11vnc_macosx_no_deprecated_framebuffer) {
-		return macosx_opengl_get_height();
-	}
-	return (int) CGDisplayPixelsHigh(displayID);
-}
 int macosxCG_CGDisplayBitsPerPixel(void) {
-	if ((0 && macosx_read_opengl) || x11vnc_macosx_no_deprecated_framebuffer) {
-		return macosx_opengl_get_bpp();
+	if (x11vnc_macosx_no_deprecated_framebuffer) {
+		return 0;
 	}
 	return (int) CGDisplayBitsPerPixel(displayID);
 }
 int macosxCG_CGDisplayBitsPerSample(void) {
-	if (macosx_read_opengl || x11vnc_macosx_no_deprecated_framebuffer) {
-		return macosx_opengl_get_bps();
+	if (x11vnc_macosx_no_deprecated_framebuffer) {
+		return 0;
 	}
 	return (int) CGDisplayBitsPerSample(displayID);
 }
 int macosxCG_CGDisplaySamplesPerPixel(void) {
-	if (macosx_read_opengl || x11vnc_macosx_no_deprecated_framebuffer) {
-		return macosx_opengl_get_spp();
+	if (x11vnc_macosx_no_deprecated_framebuffer) {
+		return 0;
 	}
 	return (int) CGDisplaySamplesPerPixel(displayID);
 }
 int macosxCG_CGDisplayBytesPerRow(void) {
-	if (macosx_read_opengl || x11vnc_macosx_no_deprecated_framebuffer) {
-		return macosx_opengl_get_width() * macosx_opengl_get_bpp()/8;
+	if (x11vnc_macosx_no_deprecated_framebuffer) {
+		return 0;
 	}
-	return (int) CGDisplayBytesPerRow(displayID);;
+	return (int) CGDisplayBytesPerRow(displayID);
 }
 
 #endif
@@ -548,7 +657,7 @@ extern int off_x, coff_x, off_y, coff_y;
 extern int debug_pointer;
 
 static void CGPostScrollWheelEvent_wr(CGWheelCount wheel_count, int wheel_distance) {
-	static int post_mode = -1, mcnt = 0;
+	static int post_mode = -1, mcnt = 0, mmax = 16;
 
 #if !X11VNC_MACOSX_NO_DEPRECATED_POSTEVENTS && !X11VNC_MACOSX_NO_DEPRECATED
 	if (post_mode < 0) {
@@ -559,7 +668,7 @@ static void CGPostScrollWheelEvent_wr(CGWheelCount wheel_count, int wheel_distan
 	}
 	
 	if (post_mode) {
-		if (mcnt++ < 10 || debug_pointer) fprintf(stderr, "CGPostScrollWheelEvent()\n");
+		if (mcnt++ < mmax || debug_pointer) fprintf(stderr, "CGPostScrollWheelEvent(count=%d, distance=%d)\n", (int) wheel_count, wheel_distance);
 		CGPostScrollWheelEvent(wheel_count, wheel_distance);
 	} else
 #endif
@@ -573,12 +682,12 @@ static void CGPostScrollWheelEvent_wr(CGWheelCount wheel_count, int wheel_distan
 			CFRelease(event);
 		}
 #endif
-		if (mcnt++ < 10 || debug_pointer) fprintf(stderr, "CGEventCreateScrollWheelEvent()\n");
+		if (mcnt++ < mmax || debug_pointer) fprintf(stderr, "CGEventCreateScrollWheelEvent(count=%d, distance=%d)\n", (int) wheel_count, wheel_distance);
 	}
 }
 
 static void CGPostMouseEvent_wr(CGPoint loc, int update, int count, int d1, int d2, int d3, int p1, int p2, int p3) {
-	static int post_mode = -1, mcnt = 0;
+	static int post_mode = -1, mcnt = 0, mmax = 16;
 
 #if !X11VNC_MACOSX_NO_DEPRECATED_POSTEVENTS && !X11VNC_MACOSX_NO_DEPRECATED
 	if (post_mode < 0) {
@@ -587,9 +696,12 @@ static void CGPostMouseEvent_wr(CGPoint loc, int update, int count, int d1, int
 			post_mode = 0;
 		}
 	}
+
+	loc.x += macosx_display_x;
+	loc.y += macosx_display_y;
 	
 	if (post_mode) {
-		if (mcnt++ < 10 || debug_pointer) fprintf(stderr, "CGPostMouseEvent()\n");
+		if (mcnt++ < mmax || debug_pointer) fprintf(stderr, "CGPostMouseEvent(%d, %d)\n", (int) loc.x, (int) loc.y);
 		CGPostMouseEvent(loc, update, count, d1, d2, d3);
 	} else
 #endif
@@ -664,7 +776,7 @@ static void CGPostMouseEvent_wr(CGPoint loc, int update, int count, int d1, int
 			}
 		}
 #endif
-		if (mcnt++ < 10 || debug_pointer) fprintf(stderr, "CGEventCreateMouseEvent()\n");
+		if (mcnt++ < mmax || debug_pointer) fprintf(stderr, "CGEventCreateMouseEvent(%d, %d)\n", (int) loc.x, (int) loc.y);
 	}
 }
 
