@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 2002-2011 Karl J. Runge <runge@karlrunge.com> 
+   Copyright (C) 2002-2013 Karl J. Runge <runge@karlrunge.com> 
    All rights reserved.
 
 This file is part of x11vnc.
@@ -44,6 +44,7 @@ void macosxCGS_unused(void) {}
 #include <ApplicationServices/ApplicationServices.h>
 #include <Cocoa/Cocoa.h>
 #include <Carbon/Carbon.h>
+#include <AvailabilityMacros.h>
 
 extern CGDirectDisplayID displayID;
 
@@ -521,6 +522,7 @@ if (db) fprintf(stderr, "cnt: %d err: %d\n", (int) _wins_mapped_cnt, err);
 }
 
 #if 1
+
 NSLock *pblock = nil;
 NSString *pbstr = nil;
 NSString *cuttext = nil;
@@ -596,7 +598,139 @@ void macosxGCS_poll_pb(void) {
 	[pool release];
    }
 }
+
+static int macosx_cglion_width = 0;
+static int macosx_cglion_height = 0;
+static int macosx_cglion_bpp = 0;
+static int macosx_cglion_Bpl = 0;
+
+int macosx_cglion_set_dpy_params(void)  {
+#ifdef MAC_OS_X_VERSION_10_6
+	CGImageRef img;
+	int nw, nh;
+	static int first = 1;
+
+	nw = CGDisplayPixelsWide(displayID);
+	nh = CGDisplayPixelsHigh(displayID);
+
+	if (first) fprintf(stderr, "macosx_cglion_init: trying to initialize cglion screen capture.\n");
+	if (first || getenv("X11VNC_DEBUG_CGLION")) {
+		fprintf(stderr, "macosx_cglion_set_dpy_params: MainDisplayID=0x%x active: %d width: %d height: %d\n",
+		    displayID, CGDisplayIsActive(displayID), nw, nh);
+	}
+	first = 0;
+
+	if (getenv("X11VNC_FORCE_CGLION")) {
+		fprintf(stderr, "macosx_cglion_set_dpy_params: forcing cglion mode\n");
+		macosx_cglion_width  = nw;
+		macosx_cglion_height = nh;
+		macosx_cglion_bpp    = atoi(getenv("X11VNC_FORCE_CGLION"));
+		if (macosx_cglion_bpp < 8) {
+			macosx_cglion_bpp = 32;
+		}
+		return 1;
+	}
+	if (nw == 0 || nh == 0) {
+		fprintf(stderr, "macosx_cglion_set_dpy_params: predict CGDisplayCreateImage will fail.\n");
+		fprintf(stderr, "macosx_cglion_set_dpy_params: not creating image for cglion.\n");
+		return 0;
+	}
+
+	img = CGDisplayCreateImage(displayID);
+
+	macosx_cglion_width = 0;
+	macosx_cglion_height = 0;
+	macosx_cglion_bpp = 0;
+	macosx_cglion_Bpl = 0;
+
+	if (img != NULL) {
+		macosx_cglion_width  = CGImageGetWidth(img);
+		macosx_cglion_height = CGImageGetHeight(img);
+		macosx_cglion_bpp    = CGImageGetBitsPerPixel(img);
+		macosx_cglion_Bpl    = CGImageGetBytesPerRow(img);
+		CGImageRelease(img);
+		return 1;
+	}
 #endif
+	return 0;
+}
+
+extern int macosx_read_cglion, macosx_no_cglion;
+
+void macosx_cglion_init(void)  {
+	macosx_read_cglion = 0;
+	if (macosx_no_cglion) {
+		return;
+	}
+	if (macosx_cglion_set_dpy_params()) {
+		macosx_read_cglion = 1;
+	}
+	fprintf(stderr, "macosx_cglion_init: use_cglion=%d: width %d, height %d, bpp %d, Bpl %d\n",
+	    macosx_read_cglion, macosx_cglion_width, macosx_cglion_height,
+	    macosx_cglion_bpp, macosx_cglion_Bpl);
+}
+
+void macosx_cglion_fini(void)  {
+	macosx_read_cglion = 0;
+}
+
+int macosx_cglion_get_width(void) {
+	macosx_cglion_set_dpy_params();
+	return macosx_cglion_width;
+}
+
+int macosx_cglion_get_height(void) {
+	macosx_cglion_set_dpy_params();
+	return macosx_cglion_height;
+}
+
+int macosx_cglion_get_Bpl(void) {
+	macosx_cglion_set_dpy_params();
+	return macosx_cglion_Bpl;
+}
+
+int macosx_cglion_get_bpp(void) {
+	macosx_cglion_set_dpy_params();
+	if (macosx_cglion_bpp > 0) {
+		return macosx_cglion_bpp;
+	} else {
+		return 32;
+	}
+}
+
+int macosx_cglion_get_bps(void) {
+	return 8;
+}
+
+int macosx_cglion_get_spp(void) {
+	return 3;
+}
+
+void macosx_copy_cglion(char *dest, int x, int y, unsigned int w, unsigned int h) {
+#ifdef MAC_OS_X_VERSION_10_6
+	CGRect r = CGRectMake(x, y, w, h);
+	CGImageRef img = CGDisplayCreateImageForRect(displayID, r);
+	CGDataProviderRef provide = CGImageGetDataProvider(img);
+	CFDataRef data = CGDataProviderCopyData(provide);
+	int img_Bpl = CGImageGetBytesPerRow(img);
+	int img_bpp = CGImageGetBitsPerPixel(img);
+	int img_width = w*(img_bpp/8);
+	const char *src = [(NSData *)data bytes];
+
+	char *dst = dest;
+	while (h--) {
+		memcpy(dst, src, img_width);
+		dst += img_width;
+		src += img_Bpl;
+	}
+	CGImageRelease(img);
+	[(id)data release];
+#else
+	if (dest && x && y && w && h) {}
+#endif
+}
+
+#endif  /* 1 */
 
 #endif	/* __APPLE__ */
 
