@@ -1,6 +1,6 @@
 #!/usr/bin/env perl
 #
-# Copyright (c) 2009-2011 by Karl J. Runge <runge@karlrunge.com>
+# Copyright (c) 2009-2012 by Karl J. Runge <runge@karlrunge.com>
 #
 # ultravnc_repeater.pl is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -452,6 +452,12 @@ while (1) {
 		my $n = sysread($sock, $buf, $size);
 		alarm(0);
 
+		my $idx = index($buf, "\0");
+		if ($idx >= 0) {
+			$buf = substr($buf, 0, $idx);
+		}
+		$buf =~ s/\s*$//;
+
 		if ($got_alarm) {
 			lprint("$prog: read timed out: $!");
 		} elsif (! defined $n) {
@@ -526,16 +532,13 @@ sub do_new_client {
 			$ID{$id}{sock} = $sock;
 		}
 	} else {
-		my $str = sprintf("%s", $buf);
-		$str =~ s/\s*$//g;
-		$str =~ s/\0*$//g;
 		my $host = '';
 		my $port = '';
-		if ($str =~ /^(.+):(\d+)$/) {
+		if ($buf =~ /^(.+):(-?\d+)/) {
 			$host = $1;
 			$port = $2;
 		} else {
-			$host = $str;
+			$host = $buf;
 			$port = 5900;
 		}
 		if ($port < 0) {
