@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 2002-2011 Karl J. Runge <runge@karlrunge.com> 
+   Copyright (C) 2002-2013 Karl J. Runge <runge@karlrunge.com> 
    All rights reserved.
 
 This file is part of x11vnc.
@@ -219,12 +219,17 @@ int macosx_noscreensaver = 0;
 int macosx_wait_for_switch = 1;
 int macosx_mouse_wheel_speed = 5;
 int macosx_console = 0;
+int macosx_display = 0;
+int macosx_display_x = 0;
+int macosx_display_y = 0;
 int macosx_swap23 = 1;
 int macosx_resize = 1;
 int macosx_icon_anim_time = 450;
+int macosx_no_cglion = 0;
 int macosx_no_opengl = 0;
 int macosx_no_rawfb = 0;
 int macosx_read_opengl = 0;
+int macosx_read_cglion = 0;
 int macosx_read_rawfb = 0;
 
 unsigned long subwin = 0x0;	/* -id, -sid */
