@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 2002-2011 Karl J. Runge <runge@karlrunge.com> 
+   Copyright (C) 2002-2013 Karl J. Runge <runge@karlrunge.com> 
    All rights reserved.
 
 This file is part of x11vnc.
@@ -2566,7 +2566,7 @@ static int do_run_cmd(char *cmd, char *create_cmd, char *users_list_save, int cr
 				    keep_unixpw_pass, cmd, line, &n);
 			} else {
 				res = su_verify(keep_unixpw_user,
-				    keep_unixpw_pass, cmd, line, &n, nodisp);
+				    keep_unixpw_pass, cmd, NULL, line, &n, nodisp);
 			}
 		}
 
@@ -2598,7 +2598,7 @@ if (db) {fprintf(stderr, "line: "); write(2, line, n); write(2, "\n", 1); fprint
 				n = sizeof(line);
 				close_exec_fds();
 				res = su_verify(keep_unixpw_user,
-				    keep_unixpw_pass, create_cmd, line, &n, nodisp);
+				    keep_unixpw_pass, create_cmd, NULL, line, &n, nodisp);
 if (db) fprintf(stderr, "c-res=%d n=%d line: '%s'\n", res, n, line);
 
 			} else {
@@ -2915,6 +2915,8 @@ void progress_client(void) {
 	}
 }
 
+void flag_file_activities(void);
+
 int wait_for_client(int *argc, char** argv, int http) {
 	/* ugh, here we go... */
 	XImage* fb_image;
@@ -2988,9 +2990,21 @@ int wait_for_client(int *argc, char** argv, int http) {
 			h = nabs(h0);
 		}
 		if (del) {
+			free(pad_geometry);
 			pad_geometry = NULL;
 		}
 	}
+	if ((w0 == -1 || h0 == -1) && getenv("FD_GEOM") != NULL) {
+		int b0;
+		char *s = getenv("FD_GEOM");
+		if (sscanf(s, "%dx%dx%d", &w0, &h0, &b0) == 3)  {
+			w = nabs(w0);
+			h = nabs(h0);
+		} else if (sscanf(s, "%dx%d", &w0, &h0) == 2)  {
+			w = nabs(w0);
+			h = nabs(h0);
+		}
+	}
 
 	/* str currently begins with a ':' */
 	if (strstr(str, ":cmd=") == str) {
@@ -3091,6 +3105,8 @@ int wait_for_client(int *argc, char** argv, int http) {
 
 	check_waitbg();
 
+	flag_file_activities();
+
 	if (vnc_redirect) {
 		vnc_redirect_loop(vnc_redirect_test, &vnc_redirect_cnt);
 	} else {
