@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 2002-2011 Karl J. Runge <runge@karlrunge.com> 
+   Copyright (C) 2002-2013 Karl J. Runge <runge@karlrunge.com> 
    All rights reserved.
 
 This file is part of x11vnc.
@@ -47,7 +47,7 @@ int xtrap_base_event_type = 0;
 int xdamage_base_event_type = 0;
 
 /*               date +'lastmod: %Y-%m-%d' */
-char lastmod[] = "0.9.13 lastmod: 2011-08-10";
+char lastmod[] = "0.9.14 lastmod: 2013-11-21";
 
 /* X display info */
 
@@ -82,10 +82,10 @@ XImage *snap = NULL;		/* the full snap fb */
 XImage *raw_fb_image = NULL;	/* the raw fb */
 
 /* corresponding shm structures */
-XShmSegmentInfo scanline_shm;
-XShmSegmentInfo fullscreen_shm;
+XShmSegmentInfo scanline_shm = {.shmid = -1};
+XShmSegmentInfo fullscreen_shm = {.shmid = -1};
+XShmSegmentInfo snaprect_shm = {.shmid = -1};
 XShmSegmentInfo *tile_row_shm;	/* for all possible row runs */
-XShmSegmentInfo snaprect_shm;
 
 /* rfb screen info */
 rfbScreenInfoPtr screen = NULL;
