@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 2002-2010 Karl J. Runge <runge@karlrunge.com> 
+   Copyright (C) 2002-2013 Karl J. Runge <runge@karlrunge.com> 
    All rights reserved.
 
 This file is part of x11vnc.
@@ -57,6 +57,7 @@ int grab_buster = 0;
 int grab_kbd = 0;
 int grab_ptr = 0;
 int grab_always = 0;
+int ungrab_both = 0;
 int grab_local = 0;
 int sync_tod_delay = 20;
 
@@ -908,6 +909,9 @@ void sync_tod_with_servertime(void) {
 	if (! ticker_atom) {
 		return;
 	}
+	if (remote_direct) {
+		return;
+	}
 
 	XSync(dpy, False);
 	while (XCheckTypedEvent(dpy, PropertyNotify, &xev)) {
