@@ -1,5 +1,5 @@
 /*
-   Copyright (C) 2002-2011 Karl J. Runge <runge@karlrunge.com> 
+   Copyright (C) 2002-2013 Karl J. Runge <runge@karlrunge.com> 
    All rights reserved.
 
 This file is part of x11vnc.
@@ -682,7 +682,12 @@ static void copy_raw_fb_24_to_32(XImage *dest, int x, int y, unsigned int w,
 }
 
 #ifdef MACOSX
-void macosx_copy_opengl(char *, int, int, unsigned int, unsigned int);
+int macosx_copy_opengl(char *, int, int, unsigned int, unsigned int);
+int macosx_copy_cglion(char *, int, int, unsigned int, unsigned int);
+int macosx_opengl_init(void);
+int macosx_opengl_fini(void);
+int macosx_cglion_init(void);
+int macosx_cglion_fini(void);
 #endif
 
 void copy_raw_fb(XImage *dest, int x, int y, unsigned int w, unsigned int h) {
@@ -692,9 +697,19 @@ void copy_raw_fb(XImage *dest, int x, int y, unsigned int w, unsigned int h) {
 	static int db = -1;
 
 #ifdef MACOSX
-	if (macosx_console && macosx_read_opengl) {
-		macosx_copy_opengl(dest->data, x, y, w, h);
-		return;
+	if (macosx_console) {
+		if (macosx_read_opengl) {
+			if (macosx_copy_opengl(dest->data, x, y, w, h)) {
+				return;
+			}
+			macosx_opengl_fini();
+			macosx_read_opengl = 0;
+			macosx_cglion_init();
+		}
+		if (macosx_read_cglion) {
+			macosx_copy_cglion(dest->data, x, y, w, h);
+			return;
+		}
 	}
 #endif
 
